float4x4 W   : World;
float4x4 WIT : WorldInverseTranspose;
float4x4 WVP : WorldViewProjection;
float4x4 VI  : ViewInverse;

texture envTexture : Environment <
    string ResourceName = "default_reflection.dds";
    string TextureType = "Cube";
>;

sampler envMap = sampler_state {
    texture = <envTexture>;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

struct AppData {
    float3 position : POSITION;
    float3 normal : NORMAL;
};

struct VertexOutput {
    float4 hPosition : POSITION;
    float3 positionWorld : TEXCOORD0;
    float3 normalWorld : TEXCOORD1;
    float3 viewVector : TEXCOORD2;
};

struct PixelOutput {
    float4 color : COLOR;
};

VertexOutput VS_Environment(AppData IN) {
    VertexOutput OUT;
    OUT.hPosition = mul(float4(IN.position, 1.0), WVP);
    OUT.positionWorld = mul(float4(IN.position, 1.0), W).xyz;
    OUT.normalWorld = normalize(mul(float4(IN.normal, 0.0), WIT).xyz);
    OUT.viewVector = normalize(VI[3].xyz - OUT.positionWorld);
    return OUT;
}

PixelOutput PS_Environment(VertexOutput IN) {
    PixelOutput OUT;
    float3 positionWorld = normalize(IN.positionWorld);
    float3 normalWorld = normalize(IN.normalWorld + sin(100.0*positionWorld.y)*float3(0.0, 0.1, 0.0));
    float3 viewVector = normalize(IN.viewVector);
    OUT.color = texCUBE(envMap, reflect(-viewVector, normalWorld));
    return OUT;
}

technique environmentTechnique {
    pass p0 {
        VertexShader = compile vs_2_0 VS_Environment();
        PixelShader  = compile ps_2_0 PS_Environment();
    }
}