float4x4 W   : World;
float4x4 WIT : WorldInverseTranspose;
float4x4 WVP : WorldViewProjection;
float4x4 VI  : ViewInverse;

texture envTexture : Environment <
    string ResourceName = "default_reflection.dds";
    string TextureType = "Cube";
>;

sampler envMap = sampler_state {
    texture = <envTexture>;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
    MIPFILTER = LINEAR;
};

texture normalTexture <
    string ResourceName = "normal.dds";
    string TextureType = "2D";
>;

sampler normalMap = sampler_state {
    texture = <normalTexture>;
    MINFILTER = LINEAR;
    MAGFILTER = LINEAR;
};

struct AppData {
   float3 P : POSITION;
   float3 N : NORMAL;
   float3 T : TANGENT;
   float3 B : BINORMAL;
   float2 UV : TEXCOORD0;
};

struct VertexOutput {
   float4 HP : POSITION;
   float2 UV : TEXCOORD0;
   float3 N : TEXCOORD1;
   float3 T : TEXCOORD2;
   float3 B : TEXCOORD3;
   float3 V : TEXCOORD4;
};

struct PixelOutput {
    float4 color : COLOR;
};

VertexOutput VS_Normal(AppData IN) {
    VertexOutput OUT;
    float4 PObject = float4(IN.P, 1.0);
    OUT.HP = mul(PObject, WVP);
    OUT.N = normalize(mul(float4(IN.N, 0.0), WIT).xyz);
    OUT.T = normalize(mul(float4(IN.T, 0.0), W).xyz);
    OUT.B = normalize(mul(float4(IN.B, 0.0), W).xyz);
    float4 PWorld = mul(PObject, W);
    OUT.V = normalize(VI[3].xyz - PWorld.xyz);
    OUT.UV = IN.UV;
    return OUT;
}

PixelOutput PS_Normal(VertexOutput IN) {
    PixelOutput OUT;
    float3 N = normalize(IN.N);
    float3 T = normalize(IN.T);
    float3 B = normalize(IN.B);
    float3 NObj = 2.0*tex2D(normalMap, IN.UV).xyz - 1.0;
    float3 nWorld = normalize(NObj.x*T+NObj.y*B+NObj.z*N);
    OUT.color = texCUBE(envMap, reflect(-IN.V, nWorld));
    return OUT;
}

technique normalTechnique {
    pass p0 {
        VertexShader = compile vs_2_0 VS_Normal();
        PixelShader  = compile ps_2_0 PS_Normal();
    }
}