float4x4 W   : World;
float4x4 WIT : WorldInverseTranspose;
float4x4 WVP : WorldViewProjection;
float4x4 VI  : ViewInverse;

float4 lightPosition : Position <
    string Object = "PointLight";
    string Space = "World";
> = {-1.0, 2.0, -2.0, 1.0};

struct AppData {
    float3 position : POSITION;
    float3 normal : NORMAL;
};

struct VertexOutput {
    float4 hPosition : POSITION;
    float4 color : COLOR0;
};

struct PixelOutput {
    float4 color : COLOR;
};

VertexOutput VS_Lighting(AppData IN) {
    VertexOutput OUT;
    OUT.hPosition = mul(float4(IN.position, 1.0), WVP);
    float3 positionWorld = mul(float4(IN.position, 1.0), W).xyz;
    float3 lightVector = normalize(lightPosition - positionWorld);
    float3 normalWorld = normalize(mul(float4(IN.normal, 0.0), WIT).xyz);
    float3 viewVector = normalize(VI[3].xyz -  positionWorld);
    float3 halfVector = normalize(viewVector + lightVector );
    float4 lighting = lit(dot(lightVector, normalWorld), dot(halfVector,  normalWorld), 40.0);
    OUT.color = float4(0.3, 1.0, 0.6, 1.0)*lighting.y + float4(1.0, 1.0, 1.0, 1.0)*lighting.z;
    return OUT;
}

PixelOutput PS_Lighting(VertexOutput IN) {
    PixelOutput OUT;
    OUT.color = IN.color;
    return OUT;
}

technique lightingTechnique {
    pass p0 {
        VertexShader = compile vs_2_0 VS_Lighting();
        PixelShader  = compile ps_2_0 PS_Lighting();
    }
}