float4x4 W   : World;
float4x4 WIT : WorldInverseTranspose;
float4x4 WVP : WorldViewProjection;
float4x4 VI  : ViewInverse;

float4 lightPosition : Position <
    string Object = "PointLight";
    string Space = "World";
> = {-1.0, 2.0, -2.0, 1.0};

struct AppData {
    float3 position : POSITION;
    float3 normal : NORMAL;
};

struct VertexOutput {
    float4 hPosition : POSITION;
    float3 positionWorld : TEXCOORD0;
    float3 lightVector : TEXCOORD1;
    float3 normalWorld : TEXCOORD2;
    float3 viewVector : TEXCOORD3;
};

struct PixelOutput {
    float4 color : COLOR;
};

VertexOutput VS_Lighting(AppData IN) {
    VertexOutput OUT;
    OUT.hPosition = mul(float4(IN.position, 1.0), WVP);
    OUT.positionWorld = mul(float4(IN.position, 1.0), W).xyz;
    OUT.lightVector = normalize(lightPosition - OUT.positionWorld);
    OUT.normalWorld = normalize(mul(float4(IN.normal, 0.0), WIT).xyz);
    OUT.viewVector = normalize(VI[3].xyz - OUT.positionWorld);
    return OUT;
}

PixelOutput PS_Lighting(VertexOutput IN) {
    PixelOutput OUT;
    float3 positionWorld = normalize(IN.positionWorld);
    float3 lightVector = normalize(IN.lightVector);
    float3 normalWorld = normalize(IN.normalWorld);
    float3 viewVector = normalize(IN.viewVector);
    float3 halfVector = normalize(viewVector + lightVector);
    float dotLN = dot(lightVector, normalWorld);
    float dotHN = dot(halfVector,  normalWorld);
    float4 lighting = lit(dotLN, dotHN, 40.0);
    OUT.color = float4(0.3, 1.0, 0.6, 1.0)*lighting.y + float4(1.0, 1.0, 1.0, 1.0)*lighting.z;
    return OUT;
}

technique lightingTechnique {
    pass p0 {
        VertexShader = compile vs_2_0 VS_Lighting();
        PixelShader  = compile ps_2_0 PS_Lighting();
    }
}